<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Teknisa\MAN\Services;
use Teknisa\MAN\Services\RelRelacaoCuponsSatService;
use Teknisa\MAN\Utilities;

class RelRelacaoCuponsSat extends CrudWithParameterBag
{
    protected $dataSourceName = 'vnd4120Y_relRelacaoCuponsSat';
    protected $parameterBagColumns = array('IDTPAMBNFCE', 'IDSITUVENDA', 'CDFILIAL', 'CDCAIXA', 'DTVENDAINI', 'DTVENDAFIN', 'NRORG', 'IDSTATUSNFCE', 'IDMODOPERACNFCE', 'NRINSCRCONS', 'TCDCAIXA', 'IDXMLSATTRANS', 'TIDXMLSATTRANS', 'TCDFILIAL', 'TDTABERCAIX', 'DTABERCAIX');
    protected $utilService;
    protected $relacaoCuponsSatService;

    public function __construct(Manager $dataSourceManager, ParameterBag $parameterBag, Utilities\Util $utilService, RelRelacaoCuponsSatService $relacaoCuponsSatService)
    {
        $this->utilService = $utilService;
        parent::__construct($dataSourceManager, $parameterBag);
        $this->relacaoCuponsSatService = $relacaoCuponsSatService;
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response)
    {
        ini_set('memory_limit', '1G');
        set_time_limit(120);
        $conditions = $this->utilService->getParamsAsArray($request);
        
        if(isset($conditions['CDFILIAL']) || isset($conditions['IDSTATUSNFCE']) || isset($conditions['IDSITUVENDA']) || isset($conditions['IDMODOPERACNFCE'])) {

            $conditions['CDFILIAL'] = implode('_', $conditions['CDFILIAL']);

            if(isset($conditions['NMFILIAL'])) {
                unset($conditions['NMFILIAL']);
            }

            if(!isset($conditions['IDTPAMBNFCE'])) {
                $conditions['IDTPAMBNFCE'] = null;
            }

            if(!isset($conditions['CDCAIXA'])) {
                $conditions['CDCAIXA'] = null;
            }

            if(isset($conditions['DTVENDA'])) {
                $conditions['DTVENDAINI'] = $conditions['DTVENDA'][0];
                $conditions['DTVENDAFIN'] = $conditions['DTVENDA'][1];

                unset($conditions['DTVENDA']);
            }

            if(isset($conditions['CDFILICAIXA']) && $conditions['CDFILICAIXA']) {
                $conditions['CDCAIXA'] = implode('_', $conditions['CDFILICAIXA']);

                $conditions['TCDCAIXA'] = 'F';
                unset($conditions['CDFILICAIXA']);
            } else {
                $conditions['CDCAIXA'] = null;
                $conditions['TCDCAIXA'] = 'T';
            }

            if(isset($conditions['DTABERCAIX']) && $conditions['DTABERCAIX']) {
                $conditions['DTABERCAIX'] = implode('_', $conditions['DTABERCAIX']);

                $conditions['TDTABERCAIX'] = 'F';
            } else {
                $conditions['DTABERCAIX'] = null;
                $conditions['TDTABERCAIX'] = 'T';
            }

            $conditions['NRINSCRCONS'] = (isset($conditions['NRINSCRCONS'])) ? $conditions['NRINSCRCONS'] : 'T';

            $conditions['IDSITUVENDA'] = count($conditions['IDSITUVENDA']) > 1 ? 'T' : $conditions['IDSITUVENDA'][0];

            if(!empty($conditions['IDSTATUSNFCE'])) {
                $conditions['IDXMLSATTRANS'] = implode('_', $conditions['IDSTATUSNFCE']);
                $conditions['TIDXMLSATTRANS'] = null;
            } else {
                $conditions['IDXMLSATTRANS'] = null;
                $conditions['TIDXMLSATTRANS'] = 'T';
            }
            $conditions['IDSTATUSNFCE'] = implode('_', $conditions['IDSTATUSNFCE']);

            $conditions['IDMODOPERACNFCE'] = count($conditions['IDMODOPERACNFCE']) > 1 ? 'T' : $conditions['IDMODOPERACNFCE'][0];
        }
        
        $dataSourceFilter = array();

        foreach($conditions as $key => $value) {
            $dataSourceFilter[] = $condition = array(
                "columnName" => $key,
                "operator" => "=",
                "value" => $value
            );
        }

        $original = $request->getFilterCriteria()->getConditions();

        for($i = 0; $i < count($dataSourceFilter); $i++) {
            for($j = 0; $j < count($original); $j++) {
                if(
                    $dataSourceFilter[$i]['columnName'] === $original[$j]['columnName'] &&
                    $dataSourceFilter[$i]['operator'] !== $original[$j]['operator']
                ) {
                    $dataSourceFilter[$i]['operator'] = $original[$j]['operator'];
                }
            }
        }
        $newRequest = new DTO\Request\Filter(
            new FilterCriteria(
                $request->getFilterCriteria()->getDataSourceName(),
                $dataSourceFilter,
                $request->getFilterCriteria()->getPage(),
                $request->getFilterCriteria()->getPageSize()
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );

        parent::find($newRequest, $response);
    }

    public function findProdutosVendidos(DTO\Request\Filter $request, DTO\Response $response)
    {
        $conditions = $this->utilService->getParamsAsArray($request);

        $response->addDataSet(new DataSet("vnd4120Y_prodVendidos_relRelacaoCuponsSat", $this->relacaoCuponsSatService->find($conditions)));

        return $response;
    }
}
